#include <stdio.h>
#include "trywin.h"
#include "WWL/XmSeparatorGadget.h"
extern "C" {
	void	exit (int);
	void	free (void*);
};

static void
do_popup_menu(Widget w, caddr_t closure, caddr_t ev)
{
	Widget popup = (Widget)closure;
	XButtonEvent *event = (XButtonEvent *)ev;

	XmMenuPosition (popup, event);
	XtManageChild(popup);
}

void
TryWindow :: Error (char* str)
{
	if (str != 0) {
		message.DialogType (XmDIALOG_ERROR);
		message.MessageString (str);
		message.Manage ();
	}
}

void
TryWindow :: Message (char* str)
{
	if (str != 0) {
		message.DialogType (XmDIALOG_INFORMATION);
		message.MessageString (str);
		message.Manage ();
	}
}
	
static void
clean_callback (Widget w, caddr_t, caddr_t)
{
	WXmMessageBox msg(w);

	msg.RemoveAllOkCallback ();
}

void
TryWindow :: Confirm (char* str, WWL_FUN cb)
{
	if (str != 0) {
		message.DialogType (XmDIALOG_QUESTION);
		message.MessageString (str);
		message.SetOkCallback (this, cb);
		message.AddOkCallback ((XtCallbackProc)clean_callback);
		message.AddCancelCallback ((XtCallbackProc)clean_callback);
		message.Manage ();
	}
}

void
TryWindow :: HandleQuit(WCallback*)
{
	Confirm ("Are you sure you want to quit ?",
		 (WWL_FUN)&TryWindow::ReallyQuit);
}

void
TryWindow :: ReallyQuit(WCallback*)
{

	exit (0);
}
  
void
TryWindow :: HandleHelp (WCallback*)
{
	printf("help selected\n");
}

void
TryWindow :: HandleNew (WCallback*)
{
	Message ("New selected");
}

void
TryWindow :: HandleSave (WCallback* w)
{
	Error ("Don't know how to save yet");
}

void
TryWindow :: HandleSaveAs (WCallback* wcb)
{
	XmFileSelectionBoxCallbackStruct *fscbs
		= (XmFileSelectionBoxCallbackStruct *)wcb->Data();
	if (fscbs->reason != XmCR_OK)
		return;
	WXmString fname (fscbs->value);

	// DoWhatYouWant (dir, fname);
	char msg[1024];
	char* f = fname;
	sprintf(msg, "Don't know how to save to %s", f);
 	Error (msg);
	free (f);
}

void
TryWindow :: HandleLoad (WCallback* wcb)
{
	XmFileSelectionBoxCallbackStruct *fscbs
		= (XmFileSelectionBoxCallbackStruct *)wcb->Data();
	if (fscbs->reason != XmCR_OK)
		Error ("File does not exist");
	WXmString fname (fscbs->value);

	char msg[1024];
	char* f = fname;
	sprintf(msg, "Don't know how to load to %s", f);
 	Error (msg);
	free (f);
}

void
TryWindow :: HandleNewWindow (WCallback* wcb)
{
	Error ("Don't know how to create a new window yet");
}

void
TryWindow :: HandleNewView (WCallback* wcb)
{
	XmSelectionBoxCallbackStruct *lcbs = (XmSelectionBoxCallbackStruct *)wcb->Data();
	WXmString str (lcbs->value);

	char message[1024];
	char* string = str;
	sprintf(message, "create a new view on %s", string);
	free (string);
}

void
TryWindow :: HandleDeleteView (WCallback*)
{
	Confirm ("Are you sure you want to delete the view ?",
		 (WWL_FUN)&TryWindow::ReallyQuit);
}

void
TryWindow :: HandleSelectAll (WCallback*)
{
	Message ("Select all");
}

void
TryWindow :: HandleCut (WCallback*)
{
	Message ("Cut");
}

void
TryWindow :: HandlePaste (WCallback*)
{
	Message ("Paste");
}

void
TryWindow :: HandleCopy (WCallback*)
{
	Message ("Copy");
}

void
TryWindow :: HandleDelete (WCallback*)
{
	Message ("Delete");
}

static Arg menu_bar_args[] = {
	{ XmNtopAttachment,			XmATTACH_FORM},
	{ XmNleftAttachment,			XmATTACH_FORM},
	{ XmNrightAttachment,			XmATTACH_FORM},
};

static Arg tools_args[] = {
	{ XmNtopAttachment,			XmATTACH_WIDGET},
	{ XmNleftAttachment,			XmATTACH_FORM},
};


static Arg systems_args[] = {
	{ XmNautoUnmanage,			True},
	{ XmNdialogStyle,			XmDIALOG_APPLICATION_MODAL},
};

static Arg load_files_args[] = {
//	{ XmNfileTypeMask,			XmFILE_REGULAR},
	{ XmNmustMatch,				True},
	{ XmNautoUnmanage,			True},
	{ XmNdialogStyle,			XmDIALOG_APPLICATION_MODAL},
};

static Arg save_files_args[] = {
//	{ XmNfileTypeMask,			XmFILE_REGULAR},
	{ XmNmustMatch,				False},
	{ XmNautoUnmanage,			True},
	{ XmNdialogStyle,			XmDIALOG_APPLICATION_MODAL},
};

static Arg message_args[] = {
	{ XmNautoUnmanage,			True},
	{ XmNdialogStyle,			XmDIALOG_APPLICATION_MODAL},
};

static Arg scrolled_args[] = {
	XmNscrollingPolicy,			XmAUTOMATIC,
};

static Arg frame_args[] = {
	{ XmNtopAttachment,			XmATTACH_WIDGET},
 	{ XmNleftAttachment,			XmATTACH_WIDGET},
 	{ XmNbottomAttachment,			XmATTACH_FORM},
 	{ XmNrightAttachment,			XmATTACH_FORM},
};

#define DEFINE_TOOL(n,a)						      \
void									      \
TryWindow :: _concat(HandleTool,n) (WCallback* wcb)			      \
{									      \
	XmToggleButtonCallbackStruct *tbcbs =				      \
		(XmToggleButtonCallbackStruct *)wcb->Data ();		      \
									      \
	if (tbcbs->reason != XmCR_VALUE_CHANGED) {			      \
		_concat(n,_toggle).State(True, True);			      \
		tools.MenuHistory (_concat(n,_toggle));			      \
	}								      \
}
#include "tools.def"

TryWindow :: TryWindow (Display* dpy)
: toplevel (XtAppCreateShell ("try", "Try", applicationShellWidgetClass,
			      dpy, NULL, 0)),
  main_window (toplevel, "main window"),
  menu_bar (main_window, "Menu Bar",
	    menu_bar_args, XtNumber(menu_bar_args)),
  file_menu (menu_bar, "File"),
  systems (WXmDialogShell (menu_bar, 1, "systems dialog"),
	   "systems", systems_args, XtNumber(systems_args)),
  load_files (WXmDialogShell (menu_bar, 1, "load_file_dialog"),
	      "load_files", load_files_args, XtNumber(load_files_args)),
  save_files (WXmDialogShell (menu_bar, 1, "save_file_dialog"),
	      "save_files", save_files_args, XtNumber(save_files_args)),
  message (WXmDialogShell (menu_bar, 1, "message_dialog"),
	   "message", message_args, XtNumber(message_args)),
  edit_menu (menu_bar, "Edit"),
  view_menu (menu_bar, "View"),
  tools_menu (menu_bar, "Tools"),
  tools (main_window, "tools", tools_args, XtNumber(tools_args)),
#define	DEFINE_TOOL(n,a) \
  _concat(n,_toggle)(Widget(0)),
#include "tools.def"
  frame (main_window, "frame", frame_args, XtNumber(frame_args)),
  scrolled (frame, "scrolled", scrolled_args, XtNumber(scrolled_args)),
  view (scrolled, "Drawing area")
{
	main_window.Width (784);
	main_window.Height (576);

	// Specify the geometry of the main form
	CXmForm(tools).TopWidget(menu_bar); // Tools is under the menu bar
	CXmForm sf(frame);		    // frame is around the view
	sf.TopWidget (menu_bar);	    // under the menu bar
	sf.LeftWidget (tools);		    // on the left of tools

	menu_bar.Manage();

	// Add the help menu button
	WXmCascadeButton help(menu_bar.AddCascadeButton("Help", 'H'));
	help.SetActivateCallback(this, (WWL_FUN)&TryWindow::HandleHelp);
	menu_bar.MenuHelpWidget(help);

	// Add the File cascade menu
	menu_bar.AddCascadeMenu("File", 'F', file_menu);
	file_menu.AddPushButton("New", this, (WWL_FUN)&TryWindow::HandleNew)
		.Accelerator("Ctrl<Key>N", "^N"); // with an accelerator
	file_menu.AddPushButton("Load",
				(XtCallbackProc)do_popup_menu,
				(caddr_t)Widget(load_files));
//	load_files.Pattern ("*.try");
	load_files.SetOkCallback (this, (WWL_FUN)&TryWindow::HandleLoad);
	file_menu.AddPushButton("Save", this, (WWL_FUN)&TryWindow::HandleSave)
		.Accelerator("Ctrl<Key>s", "^S");
	file_menu.AddPushButton("Save as ...",
				(XtCallbackProc)do_popup_menu,
				(caddr_t)Widget(save_files));
//	save_files.Pattern ("*.try");
	save_files.SetOkCallback (this, (WWL_FUN)&TryWindow::HandleSaveAs);
	WXmSeparatorGadget sg(file_menu, "separator");
	file_menu.AddPushButton("Quit", this, (WWL_FUN)&TryWindow::HandleQuit)
		.Accelerator("Ctrl<Key>q", "^Q");
	file_menu.ManageChildren(); // do not forget to manage them somewhere

	// Add the Edit cascade menu 
	menu_bar.AddCascadeMenu ("Edit", 'E', edit_menu);
	edit_menu.AddPushButton ("Select All", this, (WWL_FUN)&TryWindow::HandleSelectAll)
		.Accelerator("Ctrl<Key>A", "^A");
	edit_menu.AddPushButton ("Cut", this, (WWL_FUN)&TryWindow::HandleCut)
		.Accelerator("Ctrl<Key>X", "^X");
	edit_menu.AddPushButton ("Paste", this, (WWL_FUN)&TryWindow::HandlePaste)
		.Accelerator("Ctrl<Key>V", "^V");
	edit_menu.AddPushButton ("Copy", this, (WWL_FUN)&TryWindow::HandleCopy)
		.Accelerator("Ctrl<Key>C", "^C");
	edit_menu.AddPushButton ("Delete", this, (WWL_FUN)&TryWindow::HandleDelete)
		.Accelerator("Delete", "Del");
	edit_menu.ManageChildren ();

	// Add the view cascade menu
	menu_bar.AddCascadeMenu ("View", 'V', view_menu);
	view_menu.AddPushButton("New View...",
				(XtCallbackProc)do_popup_menu,
				(caddr_t)Widget(systems));
	view_menu.AddPushButton("New Window", this, (WWL_FUN)&TryWindow::HandleNewWindow)
		.Accelerator("Ctrl<Key>W", "^W");
	systems.SetOkCallback (this, (WWL_FUN)&TryWindow::HandleNewView);
	view_menu.AddPushButton("Delete View", this, (WWL_FUN)&TryWindow::HandleDeleteView)
		.Accelerator("Ctrl<Key>O", "^O");
	view_menu.ManageChildren ();

	menu_bar.AddCascadeMenu ("Tools", 'T', tools_menu);
#define DEFINE_TOOL(n,a)						      \
	_concat(n,_toggle) = tools.AddToggleButton(_stringify(n), this, (WWL_FUN)&TryWindow::_concat(HandleTool,n)); 	      \
	tools_menu.AddPushButton(_stringify(n), this, (WWL_FUN)&TryWindow::_concat(HandleTool,n))	    	      \
		.Accelerator("Alt<Key>" a, "^" a);
#include "tools.def"
	tools.ManageChildren ();
	tools_menu.ManageChildren ();

	frame.MarginWidth (2);
	frame.MarginHeight (2);
	frame.ShadowThickness (1);
	frame.ShadowType(XmSHADOW_OUT);
	
	view.ResizePolicy (XmRESIZE_NONE);
	view.Width(800);
	view.Height(800);
	
	menu_bar.ManageChildren();
	
	tools.Manage ();
	scrolled.Manage ();
	view.Manage();
	frame.Manage ();
	main_window.Manage();
	toplevel.Realize ();
}

TryWindow :: ~TryWindow ()
{
}


void
TryWindow :: Destroy ()
{
	toplevel.Destroy ();
	delete this;
}

int
main(Cardinal argc, char* argv[])
{
	XtAppContext	app_context;
	Display*	display;

	XtToolkitInitialize();
	app_context = XtCreateApplicationContext();
	if ((display = XtOpenDisplay (app_context, NULL, "try", "Try",
				      NULL, 0, &argc, argv)) == NULL) {
		fprintf (stderr,"\n%s:  Can't open display\n", argv[0]);
		exit(1);
	}

	TryWindow *w = new TryWindow (display);
	
	XtAppMainLoop (app_context);
}
