/* $Id: menu.cc,v 1.6 91/02/25 15:41:20 jdf Exp $ */
#include <stream.h>
#include "menu.h"
#include "WWL/XmLabelGadget.h"

/*?class WXmPopupMenu

The class \typ{WXmPopupMenu} encapsulates a standard Motif popup menu.
?*/

/*?nodoc?*/
WXmPopupMenu :: WXmPopupMenu (WComposite& father, char* name, ArgList args, Cardinal card)
: (XmCreatePopupMenu (Widget(father), name, args, card))
{
	if (name != NULL) {
		WXmLabelGadget label(widget, "title");
		label.LabelString(name);
	}
}

/*?
Adds a push button to the popup menu and associates a callback function
to this button if the \var{proc} is not NULL. An arglist can be passed
to initialize some resources.
?*/
WXmPushButtonGadget
WXmPopupMenu :: AddPushButton (char* name, XtCallbackProc proc, caddr_t closure, ArgList args, Cardinal card)
{
	WXmPushButtonGadget pb(widget, name, args, card);

	if (proc)
		pb.AddActivateCallback(proc, closure);

	return pb;
}


/*?
Adds a push button to the popup menu and associates a callback method
to this button. The method is called with the object \var{wwl}. An
arglist can be passed to initialize some resources.
?*/
WXmPushButtonGadget
WXmPopupMenu :: AddPushButton (char* name, WWL* wwl, WWL_FUN fun, ArgList args, Cardinal card)
{
	WXmPushButtonGadget pb(widget, name, args, card);

	pb.SetActivateCallback(wwl, fun);

	return pb;
}
  
/*?class WXmPulldownMenu

The class \typ{WXmPulldownMenu} encapsulates a standard Motif pulldown menu.
?*/


/*?
Adds a push button to the pulldown menu and associates a callback function
to this button if the \var{proc} is not NULL. An arglist can be passed
to initialize some resources.
?*/
WXmPushButtonGadget
WXmPulldownMenu :: AddPushButton (char* name, XtCallbackProc proc, caddr_t closure, ArgList args, Cardinal card)
{
	WXmPushButtonGadget pb(widget, name, args, card);

	if (proc)
		pb.AddActivateCallback(proc, closure);

	return pb;
}


/*?
Adds a push button to the pulldown menu and associates a callback method
to this button. The method is called with the object \var{wwl}. An
arglist can be passed to initialize some resources.
?*/
WXmPushButtonGadget
WXmPulldownMenu :: AddPushButton (char* name, WWL* wwl, WWL_FUN fun, ArgList args, Cardinal card)
{
	WXmPushButtonGadget pb(widget, name, args, card);

	pb.SetActivateCallback(wwl, fun);

	return pb;
}

/*?class WXmMenuBar

This class encapsulates a standard menu bar.
?*/


/*?
Adds a cascade button to the menu bar. A mnemonic character can also
be added.
?*/
WXmCascadeButton
WXmMenuBar :: AddCascadeButton (char* name, char mnemonic)
{
	WXmCascadeButton cb(widget, name);
	if (mnemonic) {
		cb.Mnemonic(mnemonic);
	}
	return cb;
}

#ifdef DOC
/*?
Add a cascade menu to the menu bar.
?*/
void
WXmMenuBar :: AddCascadeMenu (char* name, char mnemonic, WXmPulldownMenu& menu)
{
}

#endif
